#pragma comment(linker,"/manifestdependency:\"type='win32' "  \
        "name='" "Microsoft.VC90" ".DebugCRT' "               \
        "version='" "9.0.21022.8" "' "                        \
        "processorArchitecture='amd64' "                      \
        "publicKeyToken='" "1fc8b3b9a1e18e3b" "'\"")
