@echo off
echo cd %CD%
setlocal EnableExtensions EnableDelayedExpansion
if "%VERBOSE_CM%"=="1" (
  set C99_TO_C89_WRAP_DEBUG_LEVEL=1
  set C99_TO_C89_WRAP_SAVE_TEMPS=1
  set C99_TO_C89_WRAP_NO_LINE_DIRECTIVES=1
  set C99_TO_C89_CONV_DEBUG_LEVEL=1
)
%~dp0c99wrap.exe -keep cl %*
